#!/bin/bash

function safeexec () {
    eval "$@"
    if [ "$?" != 0 ] ; then
	echo "Error when executing \`$@'"
	exit 1
    fi
}

echo "________________________________________________________________________________"
echo "Searching for ocaml-3.11.1"
until [ -d "ocaml-3.11.1" ] ; do
    if tar xfz ocaml-3.11.1.tar.gz || tar xjf ocaml-3.11.1.tar.bz2 ; then
	break;
    fi
    echo "I can't find ocaml-3.11.1"
    echo "I will try getting http://caml.inria.fr/pub/distrib/ocaml-3.11/ocaml-3.11.1.tar.gz"
    if [ $(uname) == Darwin ] 
    then
	curl -O  http://caml.inria.fr/pub/distrib/ocaml-3.11/ocaml-3.11.1.tar.gz
    else
	wget http://caml.inria.fr/pub/distrib/ocaml-3.11/ocaml-3.11.1.tar.gz
    fi
done


echo "________________________________________________________________________________"
echo "Creating thread library"
sleep 1
safeexec cp -r ocaml-3.11.1/otherlibs/systhreads .
cd systhreads;
safeexec patch -p1 <../PATCHES/systhreads_patch;
cd ..;
echo "________________________________________________________________________________"
echo "Creating runtime"
sleep 1;
mkdir runtime
safeexec cp -r ocaml-3.11.1/asmrun runtime/
safeexec cp -r ocaml-3.11.1/byterun runtime/
cd runtime;
safeexec patch -p1 <../PATCHES/runtime_patch;
echo "________________________________________________________________________________"
cd ../ocaml-3.11.1;
echo "compiling ocaml with new code generator"
sleep 1
safeexec patch -p1 <../PATCHES/gencode_patch;
if [ -n "$FORCE_64" ]
then
    echo "FORCE 64 mode enabled"
    safeexec ./configure --prefix $PWD/../out -cc \"gcc -m64\"
else
    echo "FORCE 64 mode disabled"
    safeexec ./configure --prefix $PWD/../out
fi
safeexec cp -r config ../runtime/
cd ../systhreads;
safeexec make libthreadsnat.a
cd ../runtime;
safeexec cp ../systhreads/posix_n.o asmrun/;
make;
safeexec cd ../ocaml-3.11.1;

echo "source patched"
sleep 1

make world;
safeexec chmod -R 777 ../ocaml-3.11.1;
safeexec make world opt install;
cd ..;
safeexec cp runtime/asmrun/libasmrun.a out/lib/ocaml
safeexec cp systhreads/libthreadsnat.a out/lib/ocaml
echo "________________________________________________________________________________"
echo "compiling gc"
sleep 1
cd runtime/gcs
safeexec make
cd ../..
echo "source compiled"
echo "________________________________________________________________________________"

