#!/bin/bash

# This script is used to create an archive of OC4MC 
# gen_package.sh [optional version] 
# ex: $./gen_package.sh -alpha1     will generate oc4mc-alpha1.tar.gz
#     $./gen_package.sh             will generate oc4mc.tar.gz



function safeexec () {
    eval "$@"
    if [ "$?" != 0 ] ; then
	echo "Error when executing \`$@'"
	exit 1
    fi
}


echo "________________________________________________________________________________"
echo "Searching for ocaml-3.11.1.tar.gz"
until [ -f "ocaml-3.11.1.tar.gz" ] ; do
    if wget http://caml.inria.fr/pub/distrib/ocaml-3.11/ocaml-3.11.1.tar.gz  ; then
	break;
    fi
    echo "I can't find ocaml-3.11.1.tar.gz"
    echo "you can try getting http://caml.inria.fr/pub/distrib/ocaml-3.11/ocaml-3.11.1.tar.gz"
    exit 1
done

if [ $# -lt 1 ]
then 
    version=""
    echo "No version specified"
else
    version=$1
fi

echo "Building backup package"
tar cvzf ../oc4mc$version-backup.tar.gz\
    --exclude=*.o \
    --exclude=*.cm* \
    --exclude=*~ \
    --exclude=\#*\# \
    --exclude=*.a \
    --exclude=*.th \
    --exclude=*.nc \
    --exclude=*.bc \
    --exclude=oc4mc*.tar.gz\
     .

rm -rf ./*

tar xvzf ../oc4mc$version-backup.tar.gz



mv ocaml-3.11.1 ocaml-new
safeexec tar xvzf ocaml-3.11.1.tar.gz
rm -rf PATCHES
mkdir -p ./PATCHES
echo "________________________________________________________________________________"
echo "Creating systhread_patch"
cd systhreads;
#remove object files
make clean;
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
cd ..;
mv systhreads systhreads_new
safeexec cp -r ocaml-3.11.1/otherlibs/systhreads .
cd systhreads;
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
cd ..;
echo "!1!"
#create patch
diff -burN systhreads systhreads_new >PATCHES/systhreads_patch
rm -rf systhreads
mv systhreads_new systhreads

echo "________________________________________________________________________________"
echo "Creating runtime_patch"
cd runtime;
#remove object files
make clean;
cd asmrun;
make clean;
cd ../byterun;
make clean;
cd ../gcs;
make clean;
cd ..
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
mv config ../config
cd ..;

mv runtime runtime_new
mkdir -p runtime
safeexec cp -r ocaml-3.11.1/asmrun runtime/
safeexec cp -r ocaml-3.11.1/byterun runtime/
cd runtime;
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
cd ..;
#create patch
diff -burN runtime runtime_new >PATCHES/runtime_patch
rm -rf runtime
mv runtime_new runtime
mv config runtime/config

echo "________________________________________________________________________________"
echo "Creating gencode_patch"
cd ocaml-new;
make clean;
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
cd ../ocaml-3.11.1;
make clean;
#remove SVN/CVS generated directory
find -name CVS | xargs rm -rf
find -name .svn | xargs rm -rf
cd ..;
#create patch
diff -burN ocaml-3.11.1 ocaml-new >PATCHES/gencode_patch
rm -rf ocaml-3.11.1
mv ocaml-new ocaml-3.11.1

cd tests;
make clean
cd ..;

mkdir oc4mc
cp -r PATCHES oc4mc/
cp *.sh oc4mc/
cp -r tests oc4mc/
cp README oc4mc/
mv oc4mc oc4mc$version
tar cvzf oc4mc$version.tar.gz\
    --exclude=*.o\
    --exclude=*.cm*\
    --exclude=*~\
    --exclude=\#*\#\
    --exclude=*.a\
    --exclude=gen_package.sh\
    --exclude=*.th\
    --exclude=*.nc\
    --exclude=*.bc\
     oc4mc$version
rm -rf oc4mc$version

