

(*
 * programme de test d'oc4mc 
 *   calcul de puissance de matrices
 *)

(* fonctions de cr?ation et d'attente de fin de threads *)

let rk_create = ref (fun f x -> f x; (false, Thread.self()));;
let rk_join  = ref (fun (b,th) -> ());;


(* limitation du nombre de threads *)
let rk_nth_threads = ref 1;;

(* faut-il allouer ou non les tableaux interm?diaires*)
let rk_alloc = ref false;;

(* combien de lignes d'un tableau un thread peut-il traiter *)
let rk_nth_lines = ref 1;;


(* version sans Mutex *) 
let k_create f e = 
  if !rk_nth_threads <= 0 then 
  begin 
    f e;
    (false, Thread.self())
  end
  else 
    begin
      decr rk_nth_threads;
      let r = Thread.create f e in
      (true, r)
    end
;;

   
let k_join (is_a_thread, th) = 
  if is_a_thread then 
  begin 
    Thread.join th; 
    incr rk_nth_threads; 
    () 
  end 
;;

(* version avec Mutex *) 
let m = Mutex.create ();;

let mk_create f e = 
  Mutex.lock m;
  if !rk_nth_threads <= 0 then 
  begin 
    Mutex.unlock m; 
    f e;
    (false, Thread.self())
  end
  else 
  begin
    decr rk_nth_threads;  
    Mutex.unlock m;
    let r = Thread.create f e in 
      (true, r)
  end
;;

   
let mk_join (is_a_thread, th) = 
  if is_a_thread then 
  begin 
    Thread.join th; 
    Mutex.lock m;
    incr rk_nth_threads; 
    Mutex.unlock m; 
    () 
  end 
;;




(* -------------------------------------------------- *)



let ligne a1 a2 a3 len size nl  = 
  for i = nl to nl+size-1 do 
    let ligne_i = 
       if !rk_alloc then  Array.create len 0.0 else a3.(i) in 
    for j = 0 to len-1 do 
      for k=0 to len-1 do 
        ligne_i.(j) <- a1.(i).(k) *. a2.(k).(j) +. ligne_i.(j)
      done
    done;
    a3.(i) <- ligne_i
  done
;;

let th_mult a1 a2 = 
  let len = Array.length a1 in 
  let a3  = if !rk_alloc then Array.create len [||] 
            else Array.create_matrix len len 0.0 in
  let f = ligne a1 a2 a3 len !rk_nth_lines in 
  let all = ref [] in 
  let step = !rk_nth_lines in 
  let v = ref 0 in 
  while !v < len do 
    let (is_a_thread,th) as r  = !rk_create f !v in 
       (if is_a_thread then  all := r :: !all);
       v := !v + step
  done;
  List.iter !rk_join !all;
  a3;;

(*    (* multiplication classique *)

let mult a1 a2 = 
  let len = Array.length a1 in
  let a3 = Array.create_matrix len 0.0 in 
  for i = 0 to len-1 do  
    for j = 0 to len-1 do 
      for k=0 to len-1 do 
        a3.(i).(j) <- a1.(i).(k) *. a2.(k).(j) +. (a3.(i).(j)
      done
    done
  done;
  a3
;;
*)

let  power a n = 
  let rec power_aux n r = 
    if n = 0 then r 
    else power_aux (n-1) (th_mult r a) 
  in 
    power_aux (n-1) a
;;

let usage () = 
  Printf.printf  "usage   : th_power array_size degree nb_thread nb_line mutex_bool alloc_bool  \nexample : th_power 1000 100 8 1 0 1\n\n "
;;

let create size = 
  let a = Array.create_matrix size size 0.0 in 
  let len = Array.length a in 
  for i = 0 to len -  1 do 
    a.(i).(i) <- float_of_int i
  done;
  a;;

let main () = 
  let nargs = Array.length Sys.argv in 
    if nargs < 7 then usage()
    else 
      let array_size = int_of_string (Sys.argv.(1)) in 
      let degree   = int_of_string (Sys.argv.(2)) in 
      let nb_threads = int_of_string (Sys.argv.(3)) in 
      let nb_lines = int_of_string (Sys.argv.(4)) in 
      let mutex_bool = if Sys.argv.(5) = "0" then false else true in 
      let alloc = if Sys.argv.(6) = "0" then false else true in 


      rk_nth_threads := nb_threads ;
      rk_nth_lines := nb_lines ;
      rk_alloc := alloc;

      if mutex_bool 
        then (rk_join := mk_join; rk_create := mk_create) 
        else (rk_join := k_join; rk_create := k_create) ;

      let a = create array_size in 

      let r = power a degree in 

      Printf.printf "2^%d = %g\n" degree r.(2).(2)
;;

main();;

(* 
time ./a.out 50 100 100 0
2^100 = 1.26765e+30
2.756u 0.206s 0:02.91 101.3%	0+0k 0+0io 0pf+0w

time ./a.out 50 100 100 1
2^100 = 1.26765e+30
2.778u 0.224s 0:02.97 100.6%	0+0k 0+0io 0pf+0w

*)

