 

(*
 * programme de test d'oc4mc 
 *   calcul de puissance de matrices
 *)


let nth_threads = ref 1;;

let m = Mutex.create ()
let lock = ref (Mutex.lock)
let unlock = ref (Mutex.unlock)
let incr = ref (fun () -> !lock m; incr nth_threads; !unlock m)
let decr = ref (fun () -> !lock m; decr nth_threads; !unlock m)

(* version avec Mutex *) 
let mk_create f e = 
  !lock m;
  if !nth_threads <= 0 then 
  begin 
    !unlock m; 
    f e;
    (false, Thread.self())
  end
  else 
  begin  
    !unlock m;
    let r = Thread.create f e in 
      !decr ();
      Printf.printf "+%!";
      (true, r)
  end
;;

   
let mk_join (is_a_thread, th) = 
  if is_a_thread then 
  begin 
    Thread.join th; 
    () 
  end 
;;





(* ici on peut changer Ntread en NMthread pour utiliser la version avec Mutex *)

let rk_create = ref (fun f x -> f x; (false, Thread.self()));;
let rk_join  = ref (fun (b,th) -> ());;
let () = rk_join := mk_join; rk_create := mk_create

let ligne a1 a2 a3 len i  = 
  let ligne_i = Array.create len 0.0 in 
  for j = 0 to len-1 do 
    for k=0 to len-1 do 
      ligne_i.(j) <- a1.(i).(k) *. a2.(k).(j) +. ligne_i.(j)
    done;
    Thread.yield();
  done;
  a3.(i) <- ligne_i
;;

let th_mult a1 a2 = 
  let len = Array.length a1 in 
  let a3  = Array.create len [||] in 
(*   let f = ligne a1 a2 a3 len in  *)

  let f x = ligne a1 a2 a3 len x ; !incr () in 

  let all = ref [] in 
  for i = 0 to len-1 do 
    let r = !rk_create f i in all := r :: !all 
  done;
  List.iter !rk_join !all;
  a3;;

(*    (* multiplication classique *)

let mult a1 a2 = 
  let len = Array.length a1 in
  let a3 = Array.create_matrix len 0.0 in 
  for i = 0 to len-1 do  
    for j = 0 to len-1 do 
      for k=0 to len-1 do 
        a3.(i).(j) <- a1.(i).(k) *. a2.(k).(j) +. (a3.(i).(j)
      done
    done
  done;
  a3
;;
*)

let  power a n = 
  let rec power_aux n r = 
    if n = 0 then r 
    else power_aux (n-1) (th_mult r a) 
  in 
    power_aux (n-1) a
;;

let usage () = 
  Printf.printf  "usage   : th_power array_size degree nb_thread mutex_bool  \nexample : th_power 1000 100 8 0\n\n "
;;

let create size = 
  let a = Array.create_matrix size size 0.0 in 
  let len = Array.length a in 
  for i = 0 to len -  1 do 
    a.(i).(i) <- float_of_int i
  done;
  a;;

let main () = 
  let nargs = Array.length Sys.argv in 
    if nargs < 5 then usage()
    else 
      let array_size = int_of_string (Sys.argv.(1)) in 
      let degree   = int_of_string (Sys.argv.(2)) in 
      let nb_thread = int_of_string (Sys.argv.(3)) in 
      let mutex_bool = if Sys.argv.(4) = "0" then false else true in 

      nth_threads := nb_thread ;

      if mutex_bool 
        then () 
        else (lock := fun _ -> (); unlock := fun _ -> ()) ;

      let a = create array_size in 

      let r = power a degree in 

      Printf.printf "2^%d = %g\n" degree r.(2).(2)
;;

main();;

(* 
time ./a.out 50 100 100 0
2^100 = 1.26765e+30
2.756u 0.206s 0:02.91 101.3%	0+0k 0+0io 0pf+0w

time ./a.out 50 100 100 1
2^100 = 1.26765e+30
2.778u 0.224s 0:02.97 100.6%	0+0k 0+0io 0pf+0w

*)

